import { HttpService } from '@nestjs/axios';
import {
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Query,
  Redirect,
} from '@nestjs/common';
import { ApiQuery } from '@nestjs/swagger';
import { ConfigService } from '@nestjs/config';
import { JwtService } from '@nestjs/jwt';
import { catchError, lastValueFrom, map } from 'rxjs';

@Controller('auth')
export class AuthController {
  constructor(
    private config: ConfigService,
    private httpService: HttpService,
    private jwtService: JwtService,
  ) {}

  @Get('/vatsim')
  @ApiQuery({
    name: 'code',
    description: 'The authorization code',
    type: String,
  })
  @Redirect()
  async vatsim(@Query('code') code) {
    if (!code) {
      throw new HttpException(
        'Code not found in the query',
        HttpStatus.NOT_ACCEPTABLE,
      );
    }

    const token = await lastValueFrom(
      this.httpService
        .post(
          `${this.config.get<string>(
            'vatsim-auth.base-url',
          )}/${this.config.get<string>('vatsim-auth.token-endpoint')}`,
          {
            grant_type: 'authorization_code',
            client_id: this.config.get<string>('vatsim-auth.client-id'),
            client_secret: this.config.get<string>('vatsim-auth.client-secret'),
            redirect_uri: 'http://localhost:3000/auth/vatsim',
            code,
          },
        )
        .pipe(
          map((response) => response.data.access_token),
          catchError((err) => {
            throw new HttpException(err.response.data, err.response.status);
          }),
        ),
    );

    const userdata = await lastValueFrom(
      this.httpService
        .get(
          `${this.config.get<string>(
            'vatsim-auth.base-url',
          )}/${this.config.get<string>('vatsim-auth.user-endpoint')}`,
          {
            headers: {
              Authorization: `Bearer ${token}`,
              Accept: 'application/json',
            },
          },
        )
        .pipe(
          map((response) => response.data.data),
          catchError((err) => {
            throw new HttpException(err.response.data, err.response.status);
          }),
        ),
    );

    if (userdata.oauth.token_valid) {
      const payload = { username: userdata.cid, sub: token };
      const accessToken = this.jwtService.sign(payload);
      return {
        url: `${this.config.get<string>(
          'frontend.base-url',
        )}/${this.config.get<string>(
          'frontend.login-endpoint',
        )}?token=${accessToken}`,
      };
    } else {
      return {
        url: `${this.config.get<string>(
          'frontend.base-url',
        )}/${this.config.get<string>('frontend.login-endpoint')}`,
      };
    }
  }
}
